﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using HtmlAgilityPack;
using System.Drawing;

namespace ExcelSpreadCopyPasteV9
{
    public partial class ExcelSpreadCopyPaste : System.Web.UI.Page
    {
        // NuGet Package Manager for HTMLAgilityPack
        // http://htmlagilitypack.codeplex.com/

        string[][] stylesClasses;

        protected void Page_Load(object sender, EventArgs e)
        {
            FpSpread1.Columns.Count = 15;
            FpSpread1.Rows.Count = 50;
            FpSpread1.ActiveSheetView.PageSize = 20;
        }

        protected void SetStyles(FarPoint.Web.Spread.SheetView activeSheet, HtmlAttribute style, string className, int r, int c)
        {
            int styleIndex = -1;
            for (int s = 0; s < stylesClasses.Length; s++)
            {
                if (Array.IndexOf(stylesClasses[s], className) != -1)
                    styleIndex = s;
            }

            if (className != "")
            {
                string[] currentClassStyles = new string[stylesClasses[styleIndex].Length];
                Array.Copy(stylesClasses[styleIndex], currentClassStyles, stylesClasses[styleIndex].Length);

                SetStyleAttributes(currentClassStyles, activeSheet, r, c);
            }

            string[] styleAttributes = style.Value.Split(new string[] { ";" }, StringSplitOptions.None);
            SetStyleAttributes(styleAttributes, activeSheet, r, c);
        }

        protected void SetStyleAttributes(string[] styleAttributes, FarPoint.Web.Spread.SheetView activeSheet, int r, int c)
        {
            foreach (string attribute in styleAttributes)
            {
                string[] attributeNameValue = attribute.Split(new string[] { ":" }, StringSplitOptions.None);
                for (int v = 0; v < attributeNameValue.Length; v++)
                {
                    attributeNameValue[v] = attributeNameValue[v].Replace("\n", string.Empty);
                    attributeNameValue[v] = attributeNameValue[v].Replace(" ", string.Empty);
                    attributeNameValue[v] = attributeNameValue[v].Replace(";", string.Empty);
                }
                if (attributeNameValue[0].Contains("color"))
                {
                    activeSheet.Cells[r, c].ForeColor = GetColor(attributeNameValue[1]);
                }
                else if (attributeNameValue[0].Contains("background"))
                {
                    activeSheet.Cells[r, c].BackColor = GetColor(attributeNameValue[1]);
                }
                else if (attributeNameValue[0].Contains("font-size"))
                {
                    activeSheet.Cells[r, c].Font.Size = new FontUnit(attributeNameValue[1]);
                }
                else if (attributeNameValue[0].Contains("font-family"))
                {
                    activeSheet.Cells[r, c].Font.Name = attributeNameValue[1];
                }
                else if (attributeNameValue[0].Contains("font-weight"))
                {
                    if (Convert.ToInt32(attributeNameValue[1]) >= 700)
                        activeSheet.Cells[r, c].Font.Bold = true;
                }
                else if (attributeNameValue[0].Contains("text-align"))
                {
                    if (attributeNameValue[1].Contains("left"))
                        activeSheet.Cells[r, c].HorizontalAlign = HorizontalAlign.Left;
                    else if (attributeNameValue[1].Contains("right"))
                        activeSheet.Cells[r, c].HorizontalAlign = HorizontalAlign.Right;
                }
            }
        }

        protected Color GetColor(string color)
        {
            Color colorVar;
            if (color.StartsWith("#"))
                colorVar = ColorTranslator.FromHtml(color);
            else
                colorVar = Color.FromName(color);
            return colorVar;
        }

        protected void FpSpread1_ButtonCommand(object sender, FarPoint.Web.Spread.SpreadCommandEventArgs e)
        {
            if (e.CommandName == "PasteSpread")
            {
                FarPoint.Web.Spread.SheetView activeSheet = FpSpread1.ActiveSheetView;

                HtmlDocument doc = new HtmlDocument();
                doc.LoadHtml(Hidden1.Value);

                // Parse through styles and create an array
                HtmlNode styles = doc.DocumentNode.SelectNodes("//style")[0];
                string styleString = styles.InnerHtml.Substring(styles.InnerHtml.IndexOf(".xl") + 1);
                string[] stylesClassArray = styleString.Split(new string[] { "\n." }, StringSplitOptions.None);
                stylesClasses = new string[stylesClassArray.Length][];
                for (int s = 0; s < stylesClassArray.Length; s++)
                {
                    stylesClassArray[s] = stylesClassArray[s].Replace("{", string.Empty);
                    stylesClassArray[s] = stylesClassArray[s].Replace("}", string.Empty);
                    stylesClasses[s] = stylesClassArray[s].Split(new string[] { "\n\t" }, StringSplitOptions.None);
                }

                HtmlNode table = doc.DocumentNode.SelectNodes("//table")[0];
                HtmlNode body = table.SelectNodes("tbody")[0];

                FpSpread1.Rows.Count = body.SelectNodes("tr").Count;
                for (int r = 0; r < body.SelectNodes("tr").Count; r++)
                {
                    HtmlNode row = body.SelectNodes("tr")[r];

                    FpSpread1.Columns.Count = row.SelectNodes("th|td").Count;
                    for (int c = 0; c < row.SelectNodes("th|td").Count; c++)
                    {
                        HtmlNode cell = row.SelectNodes("th|td")[c];
                        string className = "";
                        foreach (HtmlAttribute style in cell.Attributes)
                        {
                            if (style.Name == "height")
                                activeSheet.Rows[r].Height = Convert.ToInt32(style.Value);
                            else if (style.Name == "width")
                                activeSheet.Columns[c].Width = Convert.ToInt32(style.Value);
                            else if (style.Name == "align")
                            {
                                if (style.Value.Contains("left"))
                                    activeSheet.Cells[r, c].HorizontalAlign = HorizontalAlign.Left;
                                else if (style.Value.Contains("right"))
                                    activeSheet.Cells[r, c].HorizontalAlign = HorizontalAlign.Right;
                            }
                            else if (style.Name == "class")
                            {
                                className = style.Value;
                                SetStyles(activeSheet, style, className, r, c);
                            }
                            else if (style.Name == "style")
                            {
                                SetStyles(activeSheet, style, className, r, c);
                            }
                        }
                        activeSheet.Cells[r, c].Value = cell.InnerText.Replace("&nbsp;", string.Empty);
                    }
                }
            }
        }
    }
}